@@ -33,7 +33,7 @@ from .doc_utils import (
 )
 
 
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 class DB(object):
@@ -65,11 +65,14 @@ class DB(object):
     ):
 
         # not a fan of this
-        if config != ":memory:":
+        if config != "sqlite///:memory:":
             self.config = config
         else:
             self.config = "sqlite://"
 
+        self.engine = None
+        self.session = None
+
         for key in sqlalchemy.__all__:
             self.__setattr__(key, sqlalchemy.__dict__.__getitem__(key))
 
@@ -130,6 +133,7 @@ class DB(object):
             engine (obj) : newly created engine
         """
         self.engine = create_engine(self.config, echo=echo)
+        self.engine.connect()
         return self.engine
 
     def create_session(self, checkfirst=True, echo=False, *args, **kwargs):
@@ -151,9 +155,8 @@ class DB(object):
         Returns:
             session (obj)
         """
-        engine = create_engine(self.config, echo=echo, *args, **kwargs)
+        engine = self.create_engine(self.config, echo=echo, *args, **kwargs)
         engine.connect()
-
         session = orm.sessionmaker(bind=engine)()
 
         self.Model().metadata.create_all(engine, checkfirst=checkfirst)
@@ -161,17 +164,28 @@ class DB(object):
 
         return session
 
-    def drop_all(self, echo=False):
+    def get_table_list(self):
+        """
+        This function returns a list of tables as found in the database.
+        Note that this list can be different from a list of model objects in
+        the case where the tables were created outside of the models.
+        """
+        engine = create_engine(self.config)
+        return engine.table_names()
+
+    def drop_all(self, tables=None, checkfirst=True, echo=False):
         """ drop_all
         Drop all tables and sequences.
 
-        Leaves the database empty, bereft, alone, a pale shadow of its
-        former self.
+        This function drops tables.
 
         Default:
-            drop_all(echo=False)
+            drop_all(self, tables=None, checkfirst=True, echo=False)
 
         Args:
+            tables: (None | list) : An optional list of tables, rather than all
+            checkfirst: (bool) : Can verify the existence prior to attempting
+                the drop
             echo: (bool : str) : logs interactions with engine
                 to INFO. defaults to False. echo can also be "debug" for more
                 detail.
@@ -179,7 +193,7 @@ class DB(object):
         # see how this session is not the 'session' object
         self.orm.session.close_all_sessions()
         engine = create_engine(self.config, echo=echo)
-        self.Model().metadata.drop_all(engine)
+        self.Model().metadata.drop_all(engine, tables=tables, checkfirst=checkfirst)
 
     def create_all(self, bind=None, checkfirst=True):
         """create_all
@@ -311,7 +325,7 @@ class DB(object):
             serial_fields: (None : list) : specify a limited list of columns
             serial_field_relations: (None: dict) : Can control what serial
             fields are included in relationships
-            level_limits: (None : set) : A technical variable related to
+            level_limits: (None : dict) : A technical variable related to
             preventing runaway recursion. Best to leave it alone.
             orig_cls: (None: str) : A technical variable related to recursion,
             leave this variable alone as well.
@@ -329,13 +343,15 @@ class DB(object):
             properties[key] = item_dict
 
         if level_limits is None:
-            level_limits = set()
+            level_limits = {}
             orig_cls = cls._class()
 
         if cls._class() in level_limits:
             # it has already been done
             if not cls._has_self_ref():
                 return STOP_VALUE
+            elif level_limits[cls._class()] > 1:
+                return STOP_VALUE
 
         properties = {}
         doc = {
@@ -368,7 +384,7 @@ class DB(object):
                     var_cls = value.prop.entity.class_._class()
                     if orig_cls is not None and var_cls in level_limits:
                         # avoids erroneous exclusion
-                        level_limits.remove(var_cls)
+                        level_limits.pop(var_cls)
 
                     # relationship or None
                     item_dict = _binary_expression(
