@@ -4,6 +4,7 @@ This module implements a base model to be used for table creation.
 
 """
 import json
+from inspect import signature
 from sqlalchemy import inspect
 from sqlalchemy.ext.declarative import as_declarative
 from sqlalchemy.orm.attributes import InstrumentedAttribute
@@ -274,14 +275,18 @@ class Model(object):
                 value = [item for item in value.all()]
 
             status = True
+
             if self._is_write_only(key):
                 if value is not None:
                     status = False
 
             if status:
                 if callable(value):
-                    value = value()
-
+                    if len(signature(value).parameters) == 0:
+                        value = value()
+                    else:
+                        status = False
+            if status:
                 res = _eval_value(
                     value,
                     to_camel_case,
@@ -374,11 +379,13 @@ class Model(object):
         Returns:
             data (obj) : the converted data
         """
+
         def only_column_check(key):
             if key in cls.__dict__:
                 if hasattr(cls.__dict__[key], "expression"):
                     return isinstance(
-                        cls.__dict__[key].expression, cls.db.Column)
+                        cls.__dict__[key].expression, cls.db.Column
+                    )
             return False
 
         if isinstance(data, str) or isinstance(data, bytes):
@@ -640,10 +647,11 @@ class Model(object):
         This function returns True if the column has 'writeOnly' True
         in the info field.
         """
-        tmp = cls.__dict__[column_name]
-        if isinstance(tmp, InstrumentedAttribute):
-            if isinstance(tmp.expression, cls.db.Column):
-                if "writeOnly" in tmp.expression.info:
-                    value = tmp.expression.info["writeOnly"]
-                    return value
+        if column_name in cls.__dict__:
+            tmp = cls.__dict__[column_name]
+            if isinstance(tmp, InstrumentedAttribute):
+                if isinstance(tmp.expression, cls.db.Column):
+                    if "writeOnly" in tmp.expression.info:
+                        value = tmp.expression.info["writeOnly"]
+                        return value
         return False
